package com.elitesland.org.service;

import com.elitesland.core.base.PagingVO;
import com.elitesland.org.param.OrgOuQParam;
import com.elitesland.org.param.OrgOuSaveParam;
import com.elitesland.org.vo.OrgOuVO;

import java.util.List;
import java.util.Optional;

public interface OrgOuService {

    /**
     * 检索PId下的子单位节点
     *
     * @param pId   父节点ID
     * @param param 查询条件
     * @return 子单位列表
     */
    PagingVO<OrgOuVO> searchByPId(Long pId, OrgOuQParam param);

    /**
     * 根据父节点ID，列出子单位的树形数据集合
     *
     * @param pId 父节点ID
     * @return 子单位的树形数据结构
     */
    List<OrgOuVO> listTreeByPId(Long pId);

    List<OrgOuVO> listByIds(List<Long> ids);

    /**
     * 根据单位ID，返回对应的VO记录
     *
     * @param id 单位ID
     * @return 单位VO记录
     */
    Optional<OrgOuVO> oneById(Long id);

    /**
     * 创建组织结构单位节点
     *
     * @param entity
     * @return
     */
    Long create(OrgOuSaveParam entity);

    /**
     * 根据提交的单位集合中元素顺序，重新排序，并保存到数据库
     * @param entities 排序的单元集合
     */
    void updateSort(List<OrgOuVO> entities);

    /**
     * 更新单位信息
     *
     * @param entity 单位信息
     */
    void update(OrgOuSaveParam entity);

    /**
     * 删除单位信息，以及其下所有的子单位
     * @param id 单位ID
     */
    void removeById(Long id);
}
